@@ -1,7 +1,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID ("$Id: pwdcheck.c,v 1.4 2004/11/02 18:46:30 kloczek Exp $")
+RCSID ("$Id: pwdcheck.c,v 1.6 2005/05/25 18:20:22 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include <pwd.h>
@@ -13,8 +13,7 @@ RCSID ("$Id: pwdcheck.c,v 1.4 2004/11/02 18:46:30 kloczek Exp $")
 #include "pam_defs.h"
 #endif
 #define WRONGPWD2	"incorrect password for `%s'"
-void
-passwd_check (const char *user, const char *passwd, const char *progname)
+void passwd_check (const char *user, const char *passwd, const char *progname)
 {
 #ifdef USE_PAM
 	pam_handle_t *pamh = NULL;
@@ -45,13 +44,11 @@ passwd_check (const char *user, const char *passwd, const char *progname)
 
 #else				/* !USE_PAM */
 
-#ifdef SHADOWPWD
 	struct spwd *sp;
 
 	if ((sp = getspnam (user)))
 		passwd = sp->sp_pwdp;
 	endspent ();
-#endif
 	if (pw_auth (passwd, user, PW_LOGIN, (char *) 0) != 0) {
 		SYSLOG ((LOG_WARN, WRONGPWD2, user));
 		sleep (1);
