@@ -30,21 +30,28 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: chage.c,v 1.35 2005/01/17 23:12:04 kloczek Exp $")
-#include <sys/types.h>
-#include <stdio.h>
-#include <fcntl.h>
-#include <signal.h>
+RCSID (PKG_VER "$Id: chage.c,v 1.47 2005/06/20 15:43:09 kloczek Exp $")
 #include <ctype.h>
+#include <fcntl.h>
+#include <getopt.h>
+#include <signal.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <sys/types.h>
 #include <time.h>
-#include "prototypes.h"
-#include "defines.h"
-#ifdef SHADOWPWD
 #ifdef USE_PAM
 #include <security/pam_appl.h>
 #include <security/pam_misc.h>
 #endif				/* USE_PAM */
 #include <pwd.h>
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+#include <selinux/av_permissions.h>
+#endif
+#include "prototypes.h"
+#include "defines.h"
+#include "pwio.h"
+#include "shadowio.h"
 /*
  * Global variables
  */
@@ -52,12 +59,12 @@ static char *Prog;
 
 static int
  dflg = 0,			/* set last password change date */
- Eflg = 0,			/* set account expiration date */
- Iflg = 0,			/* set password inactive after expiration */
- lflg = 0,			/* show account aging information */
- mflg = 0,			/* set minimum number of days before password change */
- Mflg = 0,			/* set maximim number of days before password change */
- Wflg = 0;			/* set expiration warning days */
+    Eflg = 0,			/* set account expiration date */
+    Iflg = 0,			/* set password inactive after expiration */
+    lflg = 0,			/* show account aging information */
+    mflg = 0,			/* set minimum number of days before password change */
+    Mflg = 0,			/* set maximim number of days before password change */
+    Wflg = 0;			/* set expiration warning days */
 
 static int locks;
 
@@ -68,24 +75,11 @@ static long warndays;
 static long inactdays;
 static long expdays;
 
-/*
- * External identifiers
- */
-
-extern long a64l ();
-extern char *l64a ();
-
-#include "pwio.h"
-
-#include "shadowio.h"
-
-#ifdef	NDBM
-extern int pw_dbm_mode;
-extern int sp_dbm_mode;
-#endif
-
 #define	EPOCH		"1969-12-31"
 
+#define E_SUCCESS	0	/* success */
+#define E_NOPERM	1	/* permission denied */
+
 /* local function prototypes */
 static void usage (void);
 static void date_to_str (char *, size_t, time_t);
@@ -113,10 +107,20 @@ int isnum (const char *s)
 
 static void usage (void)
 {
-	fprintf (stderr,
-		 _
-		 ("Usage: chage [-l] [-m min_days] [-M max_days] [-W warn]\n"
-		  "             [-I inactive] [-E expire] [-d last_day] user\n"));
+	fprintf (stderr, _("Usage: chage [options] user\n"
+			   "\n"
+			   "Options:\n"
+			   "  -d, --lastday LAST_DAY	set last password change to LAST_DAY\n"
+			   "  -E, --expiredate EXPIRE_DATE	set account expiration date to EXPIRE_DATE\n"
+			   "  -h, --help			display this help message and exit\n"
+			   "  -I, --inactive INACTIVE	set password inactive after expiration\n"
+			   "				to INACTIVE\n"
+			   "  -l, --list			show account aging information\n"
+			   "  -m, --mindays MIN_DAYS	set minimum number of days before password\n"
+			   "				change to MIN_DAYS\n"
+			   "  -M, --maxdays MAX_DAYS	set maximim number of days before password\n"
+			   "				change to MAX_DAYS\n"
+			   "  -W, --warndays WARN_DAYS	set expiration warning days to WARN_DAYS\n"));
 	exit (1);
 }
 
@@ -149,7 +153,7 @@ static int new_fields (void)
 	char *cp;
 
 	printf (_("Enter the new value, or press ENTER for the default\n"));
-	printf("\n");
+	printf ("\n");
 
 	snprintf (buf, sizeof buf, "%ld", mindays);
 	change_field (buf, sizeof buf, _("Minimum Password Age"));
@@ -165,8 +169,7 @@ static int new_fields (void)
 
 	date_to_str (buf, sizeof buf, lastday * SCALE);
 
-	change_field (buf, sizeof buf,
-		      _("Last Password Change (YYYY-MM-DD)"));
+	change_field (buf, sizeof buf, _("Last Password Change (YYYY-MM-DD)"));
 
 	if (strcmp (buf, EPOCH) == 0)
 		lastday = -1;
@@ -294,9 +297,12 @@ static void list_fields (void)
 	 * password expires that the account becomes unusable.
 	 */
 
-	printf (_("Minimum number of days between password change\t\t: %ld\n"), mindays);
-	printf (_("Maximum number of days between password change\t\t: %ld\n"), maxdays);
-	printf (_("Number of days of warning before password expires\t: %ld\n"), warndays);
+	printf (_("Minimum number of days between password change\t\t: %ld\n"),
+		mindays);
+	printf (_("Maximum number of days between password change\t\t: %ld\n"),
+		maxdays);
+	printf (_("Number of days of warning before password expires\t: %ld\n"),
+		warndays);
 }
 
 #ifdef USE_PAM
@@ -331,13 +337,13 @@ static void cleanup (int state)
  *
  *	The valid options are
  *
- *	-m	set minimum number of days before password change (*)
- *	-M	set maximim number of days before password change (*)
  *	-d	set last password change date (*)
- *	-l	show account aging information
- *	-W	set expiration warning days (*)
- *	-I	set password inactive after expiration (*)
  *	-E	set account expiration date (*)
+ *	-I	set password inactive after expiration (*)
+ *	-l	show account aging information
+ *	-M	set maximim number of days before password change (*)
+ *	-m	set minimum number of days before password change (*)
+ *	-W	set expiration warning days (*)
  *
  *	(*) requires root permission to execute.
  *
@@ -368,66 +374,78 @@ int main (int argc, char **argv)
 	textdomain (PACKAGE);
 
 	ruid = getuid ();
+#ifdef WITH_SELINUX
+	amroot = (ruid == 0 && checkPasswdAccess (PASSWD__ROOTOK) == 0);
+#else
 	amroot = (ruid == 0);
+#endif
 
 	/*
 	 * Get the program name so that error messages can use it.
 	 */
-
 	Prog = Basename (argv[0]);
 
-	OPENLOG ("chage");
+	{
+		/*
+		 * Parse the command line options.
+		 */
+		int option_index = 0;
+		int c;
+		static struct option long_options[] = {
+			{"lastday", required_argument, NULL, 'd'},
+			{"expiredate", required_argument, NULL, 'E'},
+			{"help", no_argument, NULL, 'h'},
+			{"inactive", required_argument, NULL, 'I'},
+			{"list", no_argument, NULL, 'l'},
+			{"mindays", required_argument, NULL, 'm'},
+			{"maxdays", required_argument, NULL, 'M'},
+			{"warndays", required_argument, NULL, 'W'},
+			{NULL, 0, NULL, '\0'}
+		};
 
-#ifdef	NDBM
-	sp_dbm_mode = O_RDWR;
-	pw_dbm_mode = O_RDWR;
-#endif
-
-	/*
-	 * Parse the flags. The difference between password file formats
-	 * includes the number of fields, and whether the dates are entered
-	 * as days or weeks. Shadow password file info =must= be entered in
-	 * days, while regular password file info =must= be entered in
-	 * weeks.
-	 */
-
-	while ((flag = getopt (argc, argv, "lm:M:W:I:E:d:")) != EOF) {
-		switch (flag) {
-		case 'l':
-			lflg++;
-			break;
-		case 'm':
-			mflg++;
-			mindays = strtol (optarg, 0, 10);
-			break;
-		case 'M':
-			Mflg++;
-			maxdays = strtol (optarg, 0, 10);
-			break;
-		case 'd':
-			dflg++;
-			if (!isnum (optarg))
-				lastday = strtoday (optarg);
-			else
-				lastday = strtol (optarg, 0, 10);
-			break;
-		case 'W':
-			Wflg++;
-			warndays = strtol (optarg, 0, 10);
-			break;
-		case 'I':
-			Iflg++;
-			inactdays = strtol (optarg, 0, 10);
-			break;
-		case 'E':
-			Eflg++;
-			if (!isnum (optarg))
-				expdays = strtoday (optarg);
-			else
-				expdays = strtol (optarg, 0, 10);
-			break;
-		default:
-			usage ();
+		while ((c =
+			getopt_long (argc, argv, "d:E:hI:lm:M:W:", long_options,
+				     &option_index)) != -1) {
+			switch (c) {
+			case 'd':
+				dflg++;
+				if (!isnum (optarg))
+					lastday = strtoday (optarg);
+				else
+					lastday = strtol (optarg, 0, 10);
+				break;
+			case 'E':
+				Eflg++;
+				if (!isnum (optarg))
+					expdays = strtoday (optarg);
+				else
+					expdays = strtol (optarg, 0, 10);
+				break;
+			case 'h':
+				usage ();
+				break;
+			case 'I':
+				Iflg++;
+				inactdays = strtol (optarg, 0, 10);
+				break;
+			case 'l':
+				lflg++;
+				break;
+			case 'm':
+				mflg++;
+				mindays = strtol (optarg, 0, 10);
+				break;
+			case 'M':
+				Mflg++;
+				maxdays = strtol (optarg, 0, 10);
+				break;
+			case 'W':
+				Wflg++;
+				warndays = strtol (optarg, 0, 10);
+				break;
+			default:
+				usage ();
+			}
 		}
 	}
 
@@ -443,7 +461,6 @@ int main (int argc, char **argv)
 		fprintf (stderr,
 			 _("%s: do not include \"l\" with other flags\n"),
 			 Prog);
-		closelog ();
 		usage ();
 	}
 
@@ -454,10 +471,12 @@ int main (int argc, char **argv)
 	 */
 
 	if (!amroot && !lflg) {
-		fprintf (stderr, _("%s: permission denied.\n"), Prog);
-		closelog ();
-		exit (1);
+		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
+		exit (E_NOPERM);
 	}
+
+	OPENLOG ("chage");
+
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
@@ -485,12 +504,9 @@ int main (int argc, char **argv)
 	}
 
 	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM authentication failed\n"),
-			 Prog);
-		exit (1);
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
+		exit (E_NOPERM);
 	}
-
-	OPENLOG ("chage");
 #endif				/* USE_PAM */
 
 	/*
@@ -506,19 +522,18 @@ int main (int argc, char **argv)
 	 */
 	pwrw = 0;
 	if (!pw_open (pwrw ? O_RDWR : O_RDONLY)) {
-		fprintf (stderr, _("%s: can't open password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: can't open password file\n"), Prog);
 		cleanup (1);
 		SYSLOG ((LOG_ERR, "failed opening %s", PASSWD_FILE));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (!(pw = pw_locate (argv[optind]))) {
 		fprintf (stderr, _("%s: unknown user %s\n"), Prog,
 			 argv[optind]);
 		cleanup (1);
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	pwent = *pw;
@@ -536,7 +551,7 @@ int main (int argc, char **argv)
 		cleanup (1);
 		SYSLOG ((LOG_ERR, "failed locking %s", SHADOW_FILE));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	if (!spw_open (locks ? O_RDWR : O_RDONLY)) {
 		fprintf (stderr,
@@ -544,13 +559,13 @@ int main (int argc, char **argv)
 		cleanup (2);
 		SYSLOG ((LOG_ERR, "failed opening %s", SHADOW_FILE));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	if (lflg && (setgid (getgid ()) || setuid (ruid))) {
 		fprintf (stderr, "%s: failed to drop privileges (%s)\n",
 			 Prog, strerror (errno));
-		exit (1);
+		exit (E_NOPERM);
 	}
 
 	sp = spw_locate (argv[optind]);
@@ -584,15 +599,14 @@ int main (int argc, char **argv)
 
 	if (lflg) {
 		if (!amroot && (ruid != pwent.pw_uid)) {
-			fprintf (stderr, _("%s: permission denied.\n"),
-				 Prog);
+			fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 			closelog ();
-			exit (1);
+			exit (E_NOPERM);
 		}
 		list_fields ();
 		cleanup (2);
 		closelog ();
-		exit (0);
+		exit (E_SUCCESS);
 	}
 
 	/*
@@ -601,14 +615,13 @@ int main (int argc, char **argv)
 	 */
 
 	if (!mflg && !Mflg && !dflg && !Wflg && !Iflg && !Eflg) {
-		printf (_("Changing the aging information for %s\n"),
-			name);
+		printf (_("Changing the aging information for %s\n"), name);
 		if (!new_fields ()) {
 			fprintf (stderr, _("%s: error changing fields\n"),
 				 Prog);
 			cleanup (2);
 			closelog ();
-			exit (1);
+			exit (E_NOPERM);
 		}
 	}
 	/*
@@ -628,18 +641,12 @@ int main (int argc, char **argv)
 		pwent.pw_passwd = SHADOW_PASSWD_STRING;	/* XXX warning: const */
 		if (!pw_update (&pwent)) {
 			fprintf (stderr,
-				 _("%s: can't update password file\n"),
-				 Prog);
+				 _("%s: can't update password file\n"), Prog);
 			cleanup (2);
-			SYSLOG ((LOG_ERR, "failed updating %s",
-				 PASSWD_FILE));
+			SYSLOG ((LOG_ERR, "failed updating %s", PASSWD_FILE));
 			closelog ();
-			exit (1);
+			exit (E_NOPERM);
 		}
-#ifdef NDBM
-		(void) pw_dbm_update (&pwent);
-		endpwent ();
-#endif
 	}
 
 	/*
@@ -657,79 +664,56 @@ int main (int argc, char **argv)
 
 	if (!spw_update (&spwd)) {
 		fprintf (stderr,
-			 _("%s: can't update shadow password file\n"),
-			 Prog);
+			 _("%s: can't update shadow password file\n"), Prog);
 		cleanup (2);
 		SYSLOG ((LOG_ERR, "failed updating %s", SHADOW_FILE));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
-#ifdef	NDBM
-
-	/*
-	 * See if the shadow DBM file exists and try to update it.
-	 */
-
-	if (sp_dbm_present () && !sp_dbm_update (&spwd)) {
-		fprintf (stderr,
-			 _("Error updating the DBM password entry.\n"));
-		cleanup (2);
-		SYSLOG ((LOG_ERR, "error updating DBM passwd entry"));
-		closelog ();
-		exit (1);
-	}
-	endspent ();
-
-#endif				/* NDBM */
 
 	/*
 	 * Now close the shadow password file, which will cause all of the
 	 * entries to be re-written.
 	 */
-
 	if (!spw_close ()) {
 		fprintf (stderr,
-			 _("%s: can't rewrite shadow password file\n"),
-			 Prog);
+			 _("%s: can't rewrite shadow password file\n"), Prog);
 		cleanup (2);
 		SYSLOG ((LOG_ERR, "failed rewriting %s", SHADOW_FILE));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
-
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
 
-	pampw = getpwuid(getuid());
+	pampw = getpwuid (getuid ());
 	if (pampw == NULL) {
 		retval = PAM_USER_UNKNOWN;
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval = pam_start("chage", pampw->pw_name, &conv, &pamh);
+		retval = pam_start ("chage", pampw->pw_name, &conv, &pamh);
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval = pam_authenticate(pamh, 0);
+		retval = pam_authenticate (pamh, 0);
 		if (retval != PAM_SUCCESS) {
-			pam_end(pamh, retval);
+			pam_end (pamh, retval);
 		}
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval = pam_acct_mgmt(pamh, 0);
+		retval = pam_acct_mgmt (pamh, 0);
 		if (retval != PAM_SUCCESS) {
-			pam_end(pamh, retval);
+			pam_end (pamh, retval);
 		}
 	}
 
 	if (retval != PAM_SUCCESS) {
 		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
-		exit (1);
+		exit (E_NOPERM);
 	}
-
-	OPENLOG("chage");
-#endif /* USE_PAM */
+#endif				/* USE_PAM */
 
 	/*
 	 * Close the password file. If any entries were modified, the file
@@ -737,12 +721,11 @@ int main (int argc, char **argv)
 	 */
 
 	if (!pw_close ()) {
-		fprintf (stderr, _("%s: can't rewrite password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: can't rewrite password file\n"), Prog);
 		cleanup (2);
 		SYSLOG ((LOG_ERR, "failed rewriting %s", PASSWD_FILE));
 		closelog ();
-		exit (1);
+		exit (E_NOPERM);
 	}
 	cleanup (2);
 	SYSLOG ((LOG_INFO, "changed password expiry for %s", name));
@@ -757,9 +740,8 @@ int main (int argc, char **argv)
 		}
 
 		if (retval != PAM_SUCCESS) {
-			fprintf (stderr, _("%s: PAM chauthtok failed\n"),
-				 Prog);
-			exit (1);
+			fprintf (stderr, _("%s: PAM chauthtok failed\n"), Prog);
+			exit (E_NOPERM);
 		}
 	}
 
@@ -769,18 +751,6 @@ int main (int argc, char **argv)
 #endif				/* USE_PAM */
 
 	closelog ();
-
-
-	exit (0);
+	exit (E_SUCCESS);
 	/* NOTREACHED */
 }
-
-#else				/* !SHADOWPWD */
-int main (int argc, char **argv)
-{
-	fprintf (stderr,
-		 "%s: not configured for shadow password support.\n",
-		 argv[0]);
-	exit (1);
-}
-#endif				/* !SHADOWPWD */
