@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: passwd.c,v 1.31 2005/01/17 23:12:05 kloczek Exp $")
+RCSID (PKG_VER "$Id: passwd.c,v 1.39 2005/06/20 09:36:27 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include <sys/types.h>
@@ -50,12 +50,14 @@ RCSID (PKG_VER "$Id: passwd.c,v 1.31 2005/01/17 23:12:05 kloczek Exp $")
 #endif
 #include <pwd.h>
 #include "pwauth.h"
-#ifdef SHADOWPWD
 #include "shadowio.h"
-#endif
 #include "pwio.h"
 #include "nscd.h"
 #include "getdef.h"
+#ifdef WITH_SELINUX
+#include <selinux/selinux.h>
+#include <selinux/av_permissions.h>
+#endif
 /*
  * exit status values
  */
@@ -75,20 +77,18 @@ static char *Prog;		/* Program name */
 static int amroot;		/* The real UID was 0 */
 
 static int
-#ifdef SHADOWPWD
  eflg = 0,			/* -e - force password change */
- iflg = 0,			/* -i - set inactive days */
- kflg = 0,			/* -k - change only if expired */
- nflg = 0,			/* -n - set minimum days */
- wflg = 0,			/* -w - set warning days */
- xflg = 0,			/* -x - set maximum days */
-#endif
- aflg = 0,			/* -a - show status for all users */
- dflg = 0,			/* -d - delete password */
- lflg = 0,			/* -l - lock account */
- qflg = 0,			/* -q - quiet mode */
- Sflg = 0,			/* -S - show password status */
- uflg = 0;			/* -u - unlock account */
+    iflg = 0,			/* -i - set inactive days */
+    kflg = 0,			/* -k - change only if expired */
+    nflg = 0,			/* -n - set minimum days */
+    wflg = 0,			/* -w - set warning days */
+    xflg = 0,			/* -x - set maximum days */
+    aflg = 0,			/* -a - show status for all users */
+    dflg = 0,			/* -d - delete password */
+    lflg = 0,			/* -l - lock account */
+    qflg = 0,			/* -q - quiet mode */
+    Sflg = 0,			/* -S - show password status */
+    uflg = 0;			/* -u - unlock account */
 
 /*
  * set to 1 if there are any flags which require root privileges,
@@ -96,12 +96,10 @@ static int
  */
 static int anyflag = 0;
 
-#ifdef SHADOWPWD
 static long age_min = 0;	/* Minimum days before change   */
 static long age_max = 0;	/* Maximum days until change     */
 static long warn = 0;		/* Warning days before change   */
 static long inact = 0;		/* Days without change before locked */
-#endif
 
 static int do_update_age = 0;
 
@@ -114,11 +112,6 @@ static int do_update_pwd = 0;
  * External identifiers
  */
 
-#ifdef	NDBM
-extern int sp_dbm_mode;
-extern int pw_dbm_mode;
-#endif
-
 /* local function prototypes */
 static void usage (int);
 
@@ -126,11 +119,7 @@ static void usage (int);
 static int reuse (const char *, const struct passwd *);
 static int new_password (const struct passwd *);
 
-#ifdef SHADOWPWD
 static void check_password (const struct passwd *, const struct spwd *);
-#else				/* !SHADOWPWD */
-static void check_password (const struct passwd *);
-#endif				/* !SHADOWPWD */
 static char *insert_crypt_passwd (const char *, const char *);
 #endif				/* !USE_PAM */
 static char *date_to_str (time_t);
@@ -141,9 +130,7 @@ static void oom (void);
 static char *update_crypt_pw (char *);
 static void update_noshadow (void);
 
-#ifdef SHADOWPWD
 static void update_shadow (void);
-#endif
 static long getnumber (const char *);
 
 /*
@@ -158,8 +145,7 @@ static void usage (int status)
 			 _
 			 ("       %s [-x max] [-n min] [-w warn] [-i inact] name\n"),
 			 Prog);
-		fprintf (stderr, _("       %s {-l|-u|-d|-S|-e} name\n"),
-			 Prog);
+		fprintf (stderr, _("       %s {-l|-u|-d|-S|-e} name\n"), Prog);
 	}
 	exit (status);
 }
@@ -261,8 +247,7 @@ Please use a combination of upper and lower case letters and numbers.\n"), getde
 		STRFCPY (pass, cp);
 		strzero (cp);
 
-		if (!amroot
-		    && (!obscure (orig, pass, pw) || reuse (pass, pw))) {
+		if (!amroot && (!obscure (orig, pass, pw) || reuse (pass, pw))) {
 			printf (_("Try again.\n"));
 			continue;
 		}
@@ -283,8 +268,7 @@ Please use a combination of upper and lower case letters and numbers.\n"), getde
 			return -1;
 		}
 		if (strcmp (cp, pass))
-			fprintf (stderr,
-				 _("They don't match; try again.\n"));
+			fprintf (stderr, _("They don't match; try again.\n"));
 		else {
 			strzero (cp);
 			break;
@@ -318,21 +302,12 @@ Please use a combination of upper and lower case letters and numbers.\n"), getde
  *	password for the given user.
  */
 
-#ifdef SHADOWPWD
 static void check_password (const struct passwd *pw, const struct spwd *sp)
 {
-#else
-static void check_password (const struct passwd *pw)
-{
-#endif
 	time_t now, last, ok;
 	int exp_status;
 
-#ifdef SHADOWPWD
 	exp_status = isexpired (pw, sp);
-#else
-	exp_status = isexpired (pw);
-#endif
 
 	/*
 	 * If not expired and the "change only if expired" option (idea from
@@ -350,7 +325,6 @@ static void check_password (const struct passwd *pw)
 
 	time (&now);
 
-#ifdef SHADOWPWD
 	/*
 	 * Expired accounts cannot be changed ever. Passwords which are
 	 * locked may not be changed. Passwords where min > max may not be
@@ -363,8 +337,7 @@ static void check_password (const struct passwd *pw)
 		fprintf (stderr,
 			 _("The password for %s cannot be changed.\n"),
 			 sp->sp_namp);
-		SYSLOG ((LOG_WARN, "password locked for `%s'",
-			 sp->sp_namp));
+		SYSLOG ((LOG_WARN, "password locked for `%s'", sp->sp_namp));
 		closelog ();
 		exit (E_NOPERM);
 	}
@@ -376,27 +349,12 @@ static void check_password (const struct passwd *pw)
 	last = sp->sp_lstchg * SCALE;
 	ok = last + (sp->sp_min > 0 ? sp->sp_min * SCALE : 0);
 
-#else				/* !SHADOWPWD */
-	if (pw->pw_passwd[0] == '!' || exp_status > 1) {
-		fprintf (stderr,
-			 _("The password for %s cannot be changed.\n"),
-			 pw->pw_name);
-		SYSLOG ((LOG_WARN, "password locked for `%s'",
-			 pw->pw_name));
-		closelog ();
-		exit (E_NOPERM);
-	}
-
-	last = 0;
-	ok = 0;
-#endif				/* !SHADOWPWD */
 	if (now < ok) {
 		fprintf (stderr,
 			 _
 			 ("Sorry, the password for %s cannot be changed yet.\n"),
 			 pw->pw_name);
-		SYSLOG ((LOG_WARN, "now < minimum age for `%s'",
-			 pw->pw_name));
+		SYSLOG ((LOG_WARN, "now < minimum age for `%s'", pw->pw_name));
 		closelog ();
 		exit (E_NOPERM);
 	}
@@ -442,11 +400,8 @@ static const char *pw_status (const char *pass)
 
 static void print_status (const struct passwd *pw)
 {
-#ifdef SHADOWPWD
 	struct spwd *sp;
-#endif
 
-#ifdef SHADOWPWD
 	sp = getspnam (pw->pw_name);
 	if (sp) {
 		printf ("%s %s %s %ld %ld %ld %ld\n",
@@ -457,9 +412,7 @@ static void print_status (const struct passwd *pw)
 			(sp->sp_max * SCALE) / DAY,
 			(sp->sp_warn * SCALE) / DAY,
 			(sp->sp_inact * SCALE) / DAY);
-	} else
-#endif
-	{
+	} else {
 		printf ("%s %s\n", pw->pw_name, pw_status (pw->pw_passwd));
 	}
 }
@@ -468,9 +421,7 @@ static void print_status (const struct passwd *pw)
 static void fail_exit (int status)
 {
 	pw_unlock ();
-#ifdef SHADOWPWD
 	spw_unlock ();
-#endif
 	exit (status);
 }
 
@@ -532,30 +483,18 @@ static void update_noshadow (void)
 		oom ();
 	npw->pw_passwd = update_crypt_pw (npw->pw_passwd);
 	if (!pw_update (npw)) {
-		fprintf (stderr,
-			 _("Error updating the password entry.\n"));
+		fprintf (stderr, _("Error updating the password entry.\n"));
 		SYSLOG ((LOG_ERR, "error updating password entry"));
 		fail_exit (E_FAILURE);
 	}
-#ifdef NDBM
-	if (pw_dbm_present () && !pw_dbm_update (npw)) {
-		fprintf (stderr,
-			 _("Error updating the DBM password entry.\n"));
-		SYSLOG ((LOG_ERR, "error updaring dbm password entry"));
-		fail_exit (E_FAILURE);
-	}
-	endpwent ();
-#endif
 	if (!pw_close ()) {
-		fprintf (stderr,
-			 _("Cannot commit password file changes.\n"));
+		fprintf (stderr, _("Cannot commit password file changes.\n"));
 		SYSLOG ((LOG_ERR, "can't rewrite password file"));
 		fail_exit (E_FAILURE);
 	}
 	pw_unlock ();
 }
 
-#ifdef SHADOWPWD
 static void update_shadow (void)
 {
 	const struct spwd *sp;
@@ -604,29 +543,17 @@ static void update_shadow (void)
 		nsp->sp_lstchg = 0;
 
 	if (!spw_update (nsp)) {
-		fprintf (stderr,
-			 _("Error updating the password entry.\n"));
+		fprintf (stderr, _("Error updating the password entry.\n"));
 		SYSLOG ((LOG_ERR, "error updating password entry"));
 		fail_exit (E_FAILURE);
 	}
-#ifdef NDBM
-	if (sp_dbm_present () && !sp_dbm_update (nsp)) {
-		fprintf (stderr,
-			 _("Error updating the DBM password entry.\n"));
-		SYSLOG ((LOG_ERR, "error updaring dbm password entry"));
-		fail_exit (E_FAILURE);
-	}
-	endspent ();
-#endif
 	if (!spw_close ()) {
-		fprintf (stderr,
-			 _("Cannot commit password file changes.\n"));
+		fprintf (stderr, _("Cannot commit password file changes.\n"));
 		SYSLOG ((LOG_ERR, "can't rewrite password file"));
 		fail_exit (E_FAILURE);
 	}
 	spw_unlock ();
 }
-#endif				/* SHADOWPWD */
 
 static long getnumber (const char *str)
 {
@@ -676,9 +603,7 @@ int main (int argc, char **argv)
 #ifndef USE_PAM
 	char *cp;		/* Miscellaneous character pointing  */
 
-#ifdef SHADOWPWD
 	const struct spwd *sp;	/* Shadow file entry for user   */
-#endif
 #endif
 
 	setlocale (LC_ALL, "");
@@ -744,16 +669,9 @@ int main (int argc, char **argv)
 	 * be checked for any commands which are restricted to root only.
 	 */
 
-#ifdef SHADOWPWD
-#define FLAGS "adlqr:uSekn:x:i:w:"
-#else
-# define FLAGS "adlqr:uS"
-#endif
 
-	while ((flag = getopt (argc, argv, FLAGS)) != EOF) {
-#undef FLAGS
+	while ((flag = getopt (argc, argv, "adlqr:uSekn:x:i:w:")) != EOF) {
 		switch (flag) {
-#ifdef	SHADOWPWD
 		case 'x':
 			age_max = getnumber (optarg);
 			xflg++;
@@ -784,7 +702,6 @@ int main (int argc, char **argv)
 			/* change only if expired, like Linux-PAM passwd -k. */
 			kflg++;	/* ok for users */
 			break;
-#endif				/* SHADOWPWD */
 		case 'a':
 			aflg++;
 			break;
@@ -831,8 +748,7 @@ int main (int argc, char **argv)
 	pw = get_my_pwent ();
 	if (!pw) {
 		fprintf (stderr,
-			 _("%s: Cannot determine your user name.\n"),
-			 Prog);
+			 _("%s: Cannot determine your user name.\n"), Prog);
 		exit (E_NOPERM);
 	}
 	myname = xstrdup (pw->pw_name);
@@ -850,8 +766,7 @@ int main (int argc, char **argv)
 		if (anyflag || !Sflg || (optind < argc))
 			usage (E_USAGE);
 		if (!amroot) {
-			fprintf (stderr, _("%s: permission denied.\n"),
-				 Prog);
+			fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 			exit (E_NOPERM);
 		}
 		setpwent ();
@@ -887,33 +802,36 @@ int main (int argc, char **argv)
 		usage (E_USAGE);
 
 	if (anyflag && !amroot) {
-		fprintf (stderr, _("%s: permission denied.\n"), Prog);
+		fprintf (stderr, _("%s: Permission denied.\n"), Prog);
 		exit (E_NOPERM);
 	}
-#ifdef NDBM
-	endpwent ();
-	pw_dbm_mode = O_RDWR;
-#ifdef SHADOWPWD
-	sp_dbm_mode = O_RDWR;
-#endif
-#endif
 
 	pw = getpwnam (name);
 	if (!pw) {
 		fprintf (stderr, _("%s: unknown user %s\n"), Prog, name);
 		exit (E_NOPERM);
 	}
-
+#ifdef WITH_SELINUX
 	/*
-	 * Now I have a name, let's see if the UID for the name matches the
-	 * current real UID.
+	 * If the UID of the user does not match the current real UID,
+	 * check if the change is allowed by SELinux policy.
+	 */
+	if ((pw->pw_uid != getuid ())
+	    && (checkPasswdAccess (PASSWD__PASSWD) != 0)) {
+#else
+	/*
+	 * If the UID of the user does not match the current real UID,
+	 * check if I'm root.
 	 */
-
 	if (!amroot && pw->pw_uid != getuid ()) {
+#endif
 		fprintf (stderr,
-			 _("You may not change the password for %s.\n"),
-			 name);
-		SYSLOG ((LOG_WARN, "can't change pwd for `%s'", name));
+			 _
+			 ("%s: You may not view or modify password information for %s.\n"),
+			 Prog, name);
+		SYSLOG ((LOG_WARN,
+			 "%s: can't view or modify password information for %s",
+			 Prog, name));
 		closelog ();
 		exit (E_NOPERM);
 	}
@@ -923,7 +841,6 @@ int main (int argc, char **argv)
 		exit (E_SUCCESS);
 	}
 #ifndef USE_PAM
-#ifdef SHADOWPWD
 	/*
 	 * The user name is valid, so let's get the shadow file entry.
 	 */
@@ -933,9 +850,6 @@ int main (int argc, char **argv)
 		sp = pwd_to_spwd (pw);
 
 	cp = sp->sp_pwdp;
-#else
-	cp = pw->pw_passwd;
-#endif
 
 	/*
 	 * If there are no other flags, just change the password.
@@ -949,11 +863,7 @@ int main (int argc, char **argv)
 		 * Otherwise, go ahead and set a new password.
 		 */
 
-#ifdef SHADOWPWD
 		check_password (pw, sp);
-#else
-		check_password (pw);
-#endif
 
 		/*
 		 * Let the user know whose password is being changed.
@@ -989,28 +899,23 @@ int main (int argc, char **argv)
 		do_pam_passwd (name, qflg, kflg);
 		exit (E_SUCCESS);
 	}
-#endif				/* SHADOWPWD */
+#endif				/* USE_PAM */
 	if (setuid (0)) {
 		fprintf (stderr, _("Cannot change ID to root.\n"));
 		SYSLOG ((LOG_ERR, "can't setuid(0)"));
 		closelog ();
 		exit (E_NOPERM);
 	}
-#ifdef SHADOWPWD
 	if (spw_file_present ())
 		update_shadow ();
 	else
-#endif
 		update_noshadow ();
 
 	nscd_flush_cache ("passwd");
 	nscd_flush_cache ("group");
-#ifdef SHADOWPWD
 	nscd_flush_cache ("shadow");
-#endif
 
-	SYSLOG ((LOG_INFO, "password for `%s' changed by `%s'", name,
-		 myname));
+	SYSLOG ((LOG_INFO, "password for `%s' changed by `%s'", name, myname));
 	closelog ();
 	if (!qflg)
 		printf (_("Password changed.\n"));
