@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: su.c,v 1.28 2004/11/09 03:57:36 kloczek Exp $")
+RCSID (PKG_VER "$Id: su.c,v 1.34 2005/06/20 10:17:08 kloczek Exp $")
 #include <sys/types.h>
 #include <stdio.h>
 #ifdef USE_PAM
@@ -111,7 +111,7 @@ static int iswheel (const char *username)
 {
 	struct group *grp;
 
-	grp = getgrgid (0);
+	grp = getgrnam ("wheel");;
 	if (!grp || !grp->gr_mem)
 		return 0;
 	return is_on_list (grp->gr_mem, username);
@@ -125,9 +125,8 @@ static void su_failure (const char *tty)
 #ifdef USE_SYSLOG
 	if (getdef_bool ("SYSLOG_SU_ENAB"))
 		SYSLOG ((pwent.pw_uid ? LOG_INFO : LOG_NOTICE,
-			 "- %s %s-%s", tty,
-			 oldname[0] ? oldname : "???",
-			 name[0] ? name : "???"));
+			 "- %s %s:%s", tty,
+			 oldname[0] ? oldname : "???", name[0] ? name : "???"));
 	closelog ();
 #endif
 	puts (_("Sorry."));
@@ -168,8 +167,7 @@ static void run_shell (const char *shellstr, char *args[], int doshell)
 			exit (exit_status);
 		}
 	} else if (child == -1) {
-		(void) fprintf (stderr, "%s: Cannot fork user shell\n",
-				Prog);
+		(void) fprintf (stderr, "%s: Cannot fork user shell\n", Prog);
 		SYSLOG ((LOG_WARN, "Cannot execute %s", pwent.pw_shell));
 		closelog ();
 		exit (1);
@@ -222,8 +220,7 @@ static void run_shell (const char *shellstr, char *args[], int doshell)
 	if (ret != PAM_SUCCESS) {
 		SYSLOG ((LOG_ERR, "pam_close_session: %s",
 			 pam_strerror (pamh, ret)));
-		fprintf (stderr, "%s: %s\n", Prog,
-			 pam_strerror (pamh, ret));
+		fprintf (stderr, "%s: %s\n", Prog, pam_strerror (pamh, ret));
 		pam_end (pamh, ret);
 		exit (1);
 	}
@@ -273,9 +270,8 @@ int main (int argc, char **argv)
 	RETSIGTYPE (*oldsig) ();
 	int is_console = 0;
 
-#ifdef	SHADOWPWD
 	struct spwd *spwd = 0;
-#endif
+
 #ifdef SU_ACCESS
 	char *oldpass;
 #endif
@@ -320,8 +316,7 @@ int main (int argc, char **argv)
 		 */
 		if (!amroot) {
 			fprintf (stderr,
-				 _("%s: must be run from a terminal\n"),
-				 Prog);
+				 _("%s: must be run from a terminal\n"), Prog);
 			exit (1);
 		}
 		tty = "???";
@@ -409,10 +404,8 @@ int main (int argc, char **argv)
 	 * Sort out the password of user calling su, in case needed later
 	 * -- chris
 	 */
-#ifdef SHADOWPWD
 	if ((spwd = getspnam (oldname)))
 		pw->pw_passwd = spwd->sp_pwdp;
-#endif
 	oldpass = xstrdup (pw->pw_passwd);
 #endif				/* SU_ACCESS */
 
@@ -427,13 +420,11 @@ int main (int argc, char **argv)
 
 	ret = pam_set_item (pamh, PAM_TTY, (const void *) tty);
 	if (ret == PAM_SUCCESS)
-		ret =
-		    pam_set_item (pamh, PAM_RUSER, (const void *) oldname);
+		ret = pam_set_item (pamh, PAM_RUSER, (const void *) oldname);
 	if (ret != PAM_SUCCESS) {
 		SYSLOG ((LOG_ERR, "pam_set_item: %s",
 			 pam_strerror (pamh, ret)));
-		fprintf (stderr, "%s: %s\n", Prog,
-			 pam_strerror (pamh, ret));
+		fprintf (stderr, "%s: %s\n", Prog, pam_strerror (pamh, ret));
 		pam_end (pamh, ret);
 		exit (1);
 	}
@@ -455,12 +446,10 @@ int main (int argc, char **argv)
 		exit (1);
 	}
 #ifndef USE_PAM
-#ifdef SHADOWPWD
 	spwd = NULL;
 	if (strcmp (pw->pw_passwd, SHADOW_PASSWD_STRING) == 0
 	    && (spwd = getspnam (name)))
 		pw->pw_passwd = spwd->sp_pwdp;
-#endif
 #endif				/* !USE_PAM */
 	pwent = *pw;
 
@@ -472,7 +461,7 @@ int main (int argc, char **argv)
 
 	/* The original Shadow 3.3.2 did this differently. Do it like BSD:
 	 *
-	 * - check for uid 0 instead of name "root" - there are systems with
+	 * - check for UID 0 instead of name "root" - there are systems with
 	 *   several root accounts under different names,
 	 *
 	 * - check the contents of /etc/group instead of the current group
@@ -487,8 +476,7 @@ int main (int argc, char **argv)
 		if (pwent.pw_uid == 0 && getdef_bool ("SU_WHEEL_ONLY")
 		    && !iswheel (oldname)) {
 			fprintf (stderr,
-				 _("You are not authorized to su %s\n"),
-				 name);
+				 _("You are not authorized to su %s\n"), name);
 			exit (1);
 		}
 #ifdef SU_ACCESS
@@ -504,8 +492,7 @@ int main (int argc, char **argv)
 			break;
 		default:	/* access denied (-1) or unexpected value */
 			fprintf (stderr,
-				 _("You are not authorized to su %s\n"),
-				 name);
+				 _("You are not authorized to su %s\n"), name);
 			exit (1);
 		}
 #endif				/* SU_ACCESS */
@@ -524,8 +511,7 @@ int main (int argc, char **argv)
 	if (ret != PAM_SUCCESS) {
 		SYSLOG ((LOG_ERR, "pam_authenticate: %s",
 			 pam_strerror (pamh, ret)));
-		fprintf (stderr, "%s: %s\n", Prog,
-			 pam_strerror (pamh, ret));
+		fprintf (stderr, "%s: %s\n", Prog, pam_strerror (pamh, ret));
 		pam_end (pamh, ret);
 		su_failure (tty);
 	}
@@ -572,7 +558,6 @@ int main (int argc, char **argv)
 	 */
 
 	if (!amroot) {
-#ifdef	SHADOWPWD
 		if (!spwd)
 			spwd = pwd_to_spwd (&pwent);
 
@@ -581,7 +566,6 @@ int main (int argc, char **argv)
 				 "Expired account %s", name));
 			su_failure (tty);
 		}
-#endif
 	}
 
 	/*
@@ -625,22 +609,17 @@ int main (int argc, char **argv)
 		pwent.pw_shell++;	/* skip the '*' */
 		subsystem (&pwent);	/* figure out what to execute */
 		endpwent ();
-#ifdef SHADOWPWD
 		endspent ();
-#endif
 		goto top;
 	}
 
 	sulog (tty, 1, oldname, name);	/* save SU information */
 	endpwent ();
-#ifdef SHADOWPWD
 	endspent ();
-#endif
 #ifdef USE_SYSLOG
 	if (getdef_bool ("SYSLOG_SU_ENAB"))
-		SYSLOG ((LOG_INFO, "+ %s %s-%s", tty,
-			 oldname[0] ? oldname : "???",
-			 name[0] ? name : "???"));
+		SYSLOG ((LOG_INFO, "+ %s %s:%s", tty,
+			 oldname[0] ? oldname : "???", name[0] ? name : "???"));
 #endif
 
 #ifdef USE_PAM
@@ -656,10 +635,8 @@ int main (int argc, char **argv)
 	 */
 	ret = pam_setcred (pamh, PAM_ESTABLISH_CRED);
 	if (ret != PAM_SUCCESS) {
-		SYSLOG ((LOG_ERR, "pam_setcred: %s",
-			 pam_strerror (pamh, ret)));
-		fprintf (stderr, "%s: %s\n", Prog,
-			 pam_strerror (pamh, ret));
+		SYSLOG ((LOG_ERR, "pam_setcred: %s", pam_strerror (pamh, ret)));
+		fprintf (stderr, "%s: %s\n", Prog, pam_strerror (pamh, ret));
 		pam_end (pamh, ret);
 		exit (1);
 	}
@@ -668,8 +645,7 @@ int main (int argc, char **argv)
 	if (ret != PAM_SUCCESS) {
 		SYSLOG ((LOG_ERR, "pam_open_session: %s",
 			 pam_strerror (pamh, ret)));
-		fprintf (stderr, "%s: %s\n", Prog,
-			 pam_strerror (pamh, ret));
+		fprintf (stderr, "%s: %s\n", Prog, pam_strerror (pamh, ret));
 		pam_end (pamh, ret);
 		exit (1);
 	}
