@@ -30,7 +30,7 @@
 #include <config.h>
 
 #include "rcsid.h"
-RCSID (PKG_VER "$Id: useradd.c,v 1.50 2004/11/09 02:45:43 kloczek Exp $")
+RCSID (PKG_VER "$Id: useradd.c,v 1.59 2005/06/20 10:17:09 kloczek Exp $")
 #include "prototypes.h"
 #include "defines.h"
 #include "chkname.h"
@@ -49,11 +49,7 @@ RCSID (PKG_VER "$Id: useradd.c,v 1.50 2004/11/09 02:45:43 kloczek Exp $")
 #include <pwd.h>
 #endif				/* USE_PAM */
 #include "pwauth.h"
-#if HAVE_LASTLOG_H
 #include <lastlog.h>
-#else
-#include "lastlog_.h"
-#endif
 #include "faillog.h"
 #include "nscd.h"
 #ifndef SKEL_DIR
@@ -79,10 +75,8 @@ static const char *def_shell = "";
 static const char *def_template = SKEL_DIR;
 static const char *def_create_mail_spool = "no";
 
-#ifdef SHADOWPWD
 static long def_inactive = -1;
 static const char *def_expire = "";
-#endif
 
 static char def_file[] = USER_DEFAULTS_FILE;
 
@@ -97,10 +91,9 @@ static const char *user_home = "";
 static const char *user_shell = "";
 static const char *create_mail_spool = "";
 
-#ifdef	SHADOWPWD
 static long user_expire = -1;
 static int is_shadow_pwd;
-#endif
+
 #ifdef SHADOWGRP
 static int is_shadow_grp;
 #endif
@@ -112,51 +105,25 @@ static char *Prog;
 
 static int
  bflg = 0,			/* new default root of home directory */
- cflg = 0,			/* comment (GECOS) field for new account */
- dflg = 0,			/* home directory for new account */
- Dflg = 0,			/* set/show new user default values */
- eflg = 0,			/* days since 1970-01-01 when account is locked */
- fflg = 0,			/* days until account with expired password is locked */
- gflg = 0,			/* primary group ID for new account */
- Gflg = 0,			/* secondary group set for new account */
- kflg = 0,			/* specify a directory to fill new user directory */
- mflg = 0,			/* create user's home directory if it doesn't exist */
- nflg = 0,			/* create a group having the same name as the user */
- oflg = 0,			/* permit non-unique user ID to be specified with -u */
- sflg = 0,			/* shell program for new account */
- uflg = 0;			/* specify user ID for new account */
+    cflg = 0,			/* comment (GECOS) field for new account */
+    dflg = 0,			/* home directory for new account */
+    Dflg = 0,			/* set/show new user default values */
+    eflg = 0,			/* days since 1970-01-01 when account is locked */
+    fflg = 0,			/* days until account with expired password is locked */
+    gflg = 0,			/* primary group ID for new account */
+    Gflg = 0,			/* secondary group set for new account */
+    kflg = 0,			/* specify a directory to fill new user directory */
+    mflg = 0,			/* create user's home directory if it doesn't exist */
+    nflg = 0,			/* create a group having the same name as the user */
+    oflg = 0,			/* permit non-unique user ID to be specified with -u */
+    sflg = 0,			/* shell program for new account */
+    uflg = 0;			/* specify user ID for new account */
 
 extern char *optarg;
 extern int optind;
 
-#ifdef NDBM
-extern int pw_dbm_mode;
-
-#ifdef	SHADOWPWD
-extern int sp_dbm_mode;
-#endif
-
-extern int gr_dbm_mode;
-
-#ifdef	SHADOWGRP
-extern int sg_dbm_mode;
-#endif
-#endif
-
 static int home_added;
 
-#ifdef NDBM
-static int pw_dbm_added;
-static int gr_dbm_added;
-
-#ifdef	SHADOWPWD
-static int sp_dbm_added;
-#endif
-#ifdef	SHADOWGRP
-static int sg_dbm_added;
-#endif
-#endif				/* NDBM */
-
 #include "groupio.h"
 
 #ifdef	SHADOWGRP
@@ -164,11 +131,7 @@ static int sg_dbm_added;
 #endif
 
 #include "pwio.h"
-
-#ifdef	SHADOWPWD
 #include "shadowio.h"
-#endif
-
 #include "getdef.h"
 
 /*
@@ -178,7 +141,7 @@ static int sg_dbm_added;
 #define E_PW_UPDATE	1	/* can't update password file */
 #define E_USAGE		2	/* invalid command syntax */
 #define E_BAD_ARG	3	/* invalid argument to option */
-#define E_UID_IN_USE	4	/* uid already in use (and no -o) */
+#define E_UID_IN_USE	4	/* UID already in use (and no -o) */
 #define E_NOTFOUND	6	/* specified group doesn't exist */
 #define E_NAME_IN_USE	9	/* username already in use */
 #define E_GRP_UPDATE	10	/* can't update group file */
@@ -205,10 +168,8 @@ static int get_groups (char *);
 static void usage (void);
 static void new_pwent (struct passwd *);
 
-#ifdef SHADOWPWD
 static long scale_age (long);
 static void new_spent (struct spwd *);
-#endif
 static void grp_update (void);
 static void find_new_uid (void);
 
@@ -227,33 +188,10 @@ static void create_mail (void);
 
 static void fail_exit (int code)
 {
-#ifdef NDBM
-	struct passwd pwent;
-
-	if (pw_dbm_added) {
-		pwent.pw_name = user_name;
-		pwent.pw_uid = user_id;
-		pw_dbm_remove (&pwent);
-	}
-	if (gr_dbm_added)
-		fprintf (stderr, _("%s: rebuild the group database\n"),
-			 Prog);
-#ifdef	SHADOWPWD
-	if (sp_dbm_added)
-		sp_dbm_remove (user_name);
-#endif
-#ifdef	SHADOWGRP
-	if (sg_dbm_added)
-		fprintf (stderr,
-			 _("%s: rebuild the shadow group database\n"),
-			 Prog);
-#endif
-#endif				/* NDBM */
 	if (home_added)
 		rmdir (user_home);
 
-	SYSLOG ((LOG_INFO, "failed adding user `%s', data deleted",
-		 user_name));
+	SYSLOG ((LOG_INFO, "failed adding user `%s', data deleted", user_name));
 	exit (code);
 }
 
@@ -264,7 +202,7 @@ static struct group *getgr_nam_gid (const char *name)
 	char *ep;
 
 	gid = strtoul (name, &ep, 10);
-	if (*name != '\0' && *ep == '\0')	/* valid numeric gid */
+	if (*name != '\0' && *ep == '\0')	/* valid numeric GID */
 		return getgrgid (gid);
 
 	return getgrnam (name);
@@ -280,8 +218,7 @@ static long get_number (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog,
-		 cp);
+	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -294,8 +231,7 @@ static uid_t get_uid (const char *cp)
 	if (*cp != '\0' && *ep == '\0')	/* valid number */
 		return val;
 
-	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog,
-		 cp);
+	fprintf (stderr, _("%s: invalid numeric argument `%s'\n"), Prog, cp);
 	exit (E_BAD_ARG);
 }
 
@@ -342,8 +278,7 @@ static void get_defaults (void)
 		 */
 
 		if (MATCH (buf, DGROUP)) {
-			unsigned int val =
-			    (unsigned int) strtoul (cp, &ep, 10);
+			unsigned int val = (unsigned int) strtoul (cp, &ep, 10);
 
 			if (*cp != '\0' && *ep == '\0') {	/* valid number */
 				def_group = val;
@@ -351,7 +286,7 @@ static void get_defaults (void)
 					def_gname = xstrdup (grp->gr_name);
 				} else {
 					fprintf (stderr,
-						 _("%s: unknown gid %s\n"),
+						 _("%s: unknown GID %s\n"),
 						 Prog, cp);
 				}
 			} else if ((grp = getgrnam (cp))) {
@@ -359,8 +294,7 @@ static void get_defaults (void)
 				def_gname = xstrdup (cp);
 			} else {
 				fprintf (stderr,
-					 _("%s: unknown group %s\n"), Prog,
-					 cp);
+					 _("%s: unknown group %s\n"), Prog, cp);
 			}
 		}
 
@@ -377,7 +311,6 @@ static void get_defaults (void)
 		else if (MATCH (buf, SHELL)) {
 			def_shell = xstrdup (cp);
 		}
-#ifdef SHADOWPWD
 
 		/*
 		 * Default Password Inactive value
@@ -397,7 +330,6 @@ static void get_defaults (void)
 		else if (MATCH (buf, EXPIRE)) {
 			def_expire = xstrdup (cp);
 		}
-#endif
 
 		/*
 		 * Default Skeleton information
@@ -433,10 +365,8 @@ static void show_defaults (void)
 {
 	printf ("GROUP=%u\n", (unsigned int) def_group);
 	printf ("HOME=%s\n", def_home);
-#ifdef SHADOWPWD
 	printf ("INACTIVE=%ld\n", def_inactive);
 	printf ("EXPIRE=%s\n", def_expire);
-#endif
 	printf ("SHELL=%s\n", def_shell);
 	printf ("SKEL=%s\n", def_template);
 	printf ("CREATE_MAIL_SPOOL=%s\n", def_create_mail_spool);
@@ -498,20 +428,17 @@ static int set_defaults (void)
 			*cp = '\0';
 
 		if (!out_group && MATCH (buf, DGROUP)) {
-			fprintf (ofp, DGROUP "%u\n",
-				 (unsigned int) def_group);
+			fprintf (ofp, DGROUP "%u\n", (unsigned int) def_group);
 			out_group++;
 		} else if (!out_home && MATCH (buf, HOME)) {
 			fprintf (ofp, HOME "%s\n", def_home);
 			out_home++;
-#ifdef	SHADOWPWD
 		} else if (!out_inactive && MATCH (buf, INACT)) {
 			fprintf (ofp, INACT "%ld\n", def_inactive);
 			out_inactive++;
 		} else if (!out_expire && MATCH (buf, EXPIRE)) {
 			fprintf (ofp, EXPIRE "%s\n", def_expire);
 			out_expire++;
-#endif
 		} else if (!out_shell && MATCH (buf, SHELL)) {
 			fprintf (ofp, SHELL "%s\n", def_shell);
 			out_shell++;
@@ -539,20 +466,17 @@ static int set_defaults (void)
 		fprintf (ofp, DGROUP "%u\n", (unsigned int) def_group);
 	if (!out_home)
 		fprintf (ofp, HOME "%s\n", def_home);
-#ifdef	SHADOWPWD
 	if (!out_inactive)
 		fprintf (ofp, INACT "%ld\n", def_inactive);
 	if (!out_expire)
 		fprintf (ofp, EXPIRE "%s\n", def_expire);
-#endif
 	if (!out_shell)
 		fprintf (ofp, SHELL "%s\n", def_shell);
 	if (!out_skel)
 		fprintf (ofp, SKEL "%s\n", def_template);
 
 	if (!out_create_mail_spool)
-		fprintf (ofp, CREATE_MAIL_SPOOL "%s\n",
-			 def_create_mail_spool);
+		fprintf (ofp, CREATE_MAIL_SPOOL "%s\n", def_create_mail_spool);
 
 	/*
 	 * Flush and close the file. Check for errors to make certain
@@ -571,8 +495,7 @@ static int set_defaults (void)
 
 	snprintf (buf, sizeof buf, "%s-", def_file);
 	if (rename (def_file, buf) && errno != ENOENT) {
-		snprintf (buf, sizeof buf, _("%s: rename: %s"), Prog,
-			  def_file);
+		snprintf (buf, sizeof buf, _("%s: rename: %s"), Prog, def_file);
 		perror (buf);
 		unlink (new_file);
 		return -1;
@@ -583,25 +506,16 @@ static int set_defaults (void)
 	 */
 
 	if (rename (new_file, def_file)) {
-		snprintf (buf, sizeof buf, _("%s: rename: %s"), Prog,
-			  new_file);
+		snprintf (buf, sizeof buf, _("%s: rename: %s"), Prog, new_file);
 		perror (buf);
 		return -1;
 	}
-#ifdef SHADOWPWD
 	SYSLOG ((LOG_INFO,
 		 "useradd defaults: GROUP=%u, HOME=%s, SHELL=%s, INACTIVE=%ld, "
 		 "EXPIRE=%s, SKEL=%s, CREATE_MAIL_SPOOL=%s",
 		 (unsigned int) def_group, def_home, def_shell,
 		 def_inactive, def_expire, def_template,
 		 def_create_mail_spool));
-#else
-	SYSLOG ((LOG_INFO,
-		 "useradd defaults: GROUP=%u, HOME=%s, SHELL=%s, "
-		 "SKEL=%s, CREATE_MAIL_SPOOL=%s",
-		 (unsigned int) def_group, def_home, def_shell,
-		 def_template, def_create_mail_spool));
-#endif
 	return 0;
 }
 
@@ -722,16 +636,11 @@ static void usage (void)
 	fprintf (stderr,
 		 _
 		 ("               [-d home] [-s shell] [-c comment] [-m [-k template]]\n"));
-#ifdef SHADOWPWD
 	fprintf (stderr, _("               [-f inactive] [-e expire]\n"));
-#endif
 	fprintf (stderr, _("               [-p passwd] name\n"));
 	fprintf (stderr,
 		 _("       useradd -D [-g group] [-b base] [-s shell]\n"));
-#ifdef SHADOWPWD
 	fprintf (stderr, _("               [-f inactive] [-e expire]\n"));
-#endif
-
 	exit (E_USAGE);
 }
 
@@ -746,11 +655,9 @@ static void new_pwent (struct passwd *pwent)
 {
 	memzero (pwent, sizeof *pwent);
 	pwent->pw_name = (char *) user_name;
-#ifdef SHADOWPWD
 	if (is_shadow_pwd)
 		pwent->pw_passwd = (char *) SHADOW_PASSWD_STRING;
 	else
-#endif
 		pwent->pw_passwd = (char *) user_pass;
 
 	pwent->pw_uid = user_id;
@@ -760,7 +667,6 @@ static void new_pwent (struct passwd *pwent)
 	pwent->pw_shell = (char *) user_shell;
 }
 
-#ifdef SHADOWPWD
 static long scale_age (long x)
 {
 	if (x <= 0)
@@ -789,7 +695,6 @@ static void new_spent (struct spwd *spent)
 	spent->sp_expire = scale_age (user_expire);
 	spent->sp_flag = -1;
 }
-#endif
 
 /*
  * grp_update - add user to secondary group set
@@ -814,13 +719,11 @@ static void grp_update (void)
 	 */
 
 	if (!gr_lock ()) {
-		fprintf (stderr, _("%s: error locking group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error locking group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 	if (!gr_open (O_RDWR)) {
-		fprintf (stderr, _("%s: error opening group file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: error opening group file\n"), Prog);
 		fail_exit (E_GRP_UPDATE);
 	}
 #ifdef	SHADOWGRP
@@ -869,29 +772,12 @@ static void grp_update (void)
 		ngrp->gr_mem = add_list (ngrp->gr_mem, user_name);
 		if (!gr_update (ngrp)) {
 			fprintf (stderr,
-				 "%s: error adding new group entry\n",
-				 Prog);
+				 "%s: error adding new group entry\n", Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
-#ifdef	NDBM
-		/*
-		 * Update the DBM group file with the new entry as well.
-		 */
-
-		if (!gr_dbm_update (ngrp)) {
-			fprintf (stderr,
-				 "%s: cannot add new dbm group entry\n",
-				 Prog);
-			fail_exit (E_GRP_UPDATE);
-		} else
-			gr_dbm_added++;
-#endif
 		SYSLOG ((LOG_INFO, "add `%s' to group `%s'",
 			 user_name, ngrp->gr_name));
 	}
-#ifdef NDBM
-	endgrent ();
-#endif
 
 #ifdef	SHADOWGRP
 	if (!is_shadow_grp)
@@ -934,29 +820,12 @@ static void grp_update (void)
 		nsgrp->sg_mem = add_list (nsgrp->sg_mem, user_name);
 		if (!sgr_update (nsgrp)) {
 			fprintf (stderr,
-				 _("%s: error adding new group entry\n"),
-				 Prog);
+				 _("%s: error adding new group entry\n"), Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
-#ifdef	NDBM
-		/*
-		 * Update the DBM group file with the new entry as well.
-		 */
-
-		if (!sg_dbm_update (nsgrp)) {
-			fprintf (stderr,
-				 _("%s: cannot add new dbm group entry\n"),
-				 Prog);
-			fail_exit (E_GRP_UPDATE);
-		} else
-			sg_dbm_added++;
-#endif				/* NDBM */
 		SYSLOG ((LOG_INFO, "add `%s' to shadow group `%s'",
 			 user_name, nsgrp->sg_name));
 	}
-#ifdef NDBM
-	endsgent ();
-#endif				/* NDBM */
 #endif				/* SHADOWGRP */
 }
 
@@ -1003,7 +872,7 @@ static void find_new_uid (void)
 			exit (E_NAME_IN_USE);
 		}
 		if (uflg && user_id == pwd->pw_uid) {
-			fprintf (stderr, _("%s: uid %u is not unique\n"),
+			fprintf (stderr, _("%s: UID %u is not unique\n"),
 				 Prog, (unsigned int) user_id);
 			exit (E_UID_IN_USE);
 		}
@@ -1014,10 +883,10 @@ static void find_new_uid (void)
 		}
 	}
 	/*
-	 * If a user with uid equal to UID_MAX exists, the above algorithm
+	 * If a user with UID equal to UID_MAX exists, the above algorithm
 	 * will give us UID_MAX+1 even if not unique. Search for the first
-	 * free uid starting with UID_MIN (it's O(n*n) but can be avoided
-	 * by not having users with uid equal to UID_MAX).  --marekm
+	 * free UID starting with UID_MIN (it's O(n*n) but can be avoided
+	 * by not having users with UID equal to UID_MAX).  --marekm
 	 */
 	if (!uflg && user_id == uid_max + 1) {
 		for (user_id = uid_min; user_id < uid_max; user_id++) {
@@ -1033,8 +902,7 @@ static void find_new_uid (void)
 #endif
 		}
 		if (user_id == uid_max) {
-			fprintf (stderr, _("%s: can't get unique uid\n"),
-				 Prog);
+			fprintf (stderr, _("%s: can't get unique UID\n"), Prog);
 			fail_exit (E_UID_IN_USE);
 		}
 	}
@@ -1055,13 +923,8 @@ static void process_flags (int argc, char **argv)
 	int arg;
 	char *cp;
 
-#ifdef SHADOWPWD
-#define FLAGS "A:Du:og:G:d:s:c:mk:p:f:e:b:O:M"
-#else
-#define FLAGS "A:Du:og:G:d:s:c:mk:p:b:O:M"
-#endif
-	while ((arg = getopt (argc, argv, FLAGS)) != EOF) {
-#undef FLAGS
+	while ((arg =
+		getopt (argc, argv, "A:Du:og:G:d:s:c:mk:p:f:e:b:O:M")) != EOF) {
 		switch (arg) {
 		case 'b':
 			if (!Dflg)
@@ -1103,7 +966,6 @@ static void process_flags (int argc, char **argv)
 				usage ();
 			Dflg++;
 			break;
-#ifdef SHADOWPWD
 		case 'e':
 			if (*optarg) {
 				user_expire = strtoday (optarg);
@@ -1145,7 +1007,6 @@ static void process_flags (int argc, char **argv)
 			}
 			fflg++;
 			break;
-#endif
 		case 'g':
 			grp = getgr_nam_gid (optarg);
 			if (!grp) {
@@ -1272,10 +1133,8 @@ static void process_flags (int argc, char **argv)
 		}
 	}
 
-#ifdef SHADOWPWD
 	if (!eflg)
 		user_expire = strtoday (def_expire);
-#endif
 
 	if (!gflg)
 		user_gid = def_group;
@@ -1294,23 +1153,18 @@ static void process_flags (int argc, char **argv)
 static void close_files (void)
 {
 	if (!pw_close ()) {
-		fprintf (stderr, _("%s: cannot rewrite password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: cannot rewrite password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd && !spw_close ()) {
 		fprintf (stderr,
-			 _("%s: cannot rewrite shadow password file\n"),
-			 Prog);
+			 _("%s: cannot rewrite shadow password file\n"), Prog);
 		fail_exit (E_PW_UPDATE);
 	}
-#endif
 	if (do_grp_update) {
 		if (!gr_close ()) {
 			fprintf (stderr,
-				 _("%s: cannot rewrite group file\n"),
-				 Prog);
+				 _("%s: cannot rewrite group file\n"), Prog);
 			fail_exit (E_GRP_UPDATE);
 		}
 		gr_unlock ();
@@ -1326,10 +1180,8 @@ static void close_files (void)
 			sgr_unlock ();
 #endif
 	}
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd)
 		spw_unlock ();
-#endif
 	pw_unlock ();
 }
 
@@ -1342,33 +1194,27 @@ static void close_files (void)
 static void open_files (void)
 {
 	if (!pw_lock ()) {
-		fprintf (stderr, _("%s: unable to lock password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: unable to lock password file\n"), Prog);
 		exit (E_PW_UPDATE);
 	}
 	if (!pw_open (O_RDWR)) {
-		fprintf (stderr, _("%s: unable to open password file\n"),
-			 Prog);
+		fprintf (stderr, _("%s: unable to open password file\n"), Prog);
 		pw_unlock ();
 		exit (E_PW_UPDATE);
 	}
-#ifdef	SHADOWPWD
 	if (is_shadow_pwd && !spw_lock ()) {
 		fprintf (stderr,
-			 _("%s: cannot lock shadow password file\n"),
-			 Prog);
+			 _("%s: cannot lock shadow password file\n"), Prog);
 		pw_unlock ();
 		exit (E_PW_UPDATE);
 	}
 	if (is_shadow_pwd && !spw_open (O_RDWR)) {
 		fprintf (stderr,
-			 _("%s: cannot open shadow password file\n"),
-			 Prog);
+			 _("%s: cannot open shadow password file\n"), Prog);
 		spw_unlock ();
 		pw_unlock ();
 		exit (E_PW_UPDATE);
 	}
-#endif
 }
 
 
@@ -1410,10 +1256,7 @@ static void lastlog_reset (uid_t uid)
 static void usr_update (void)
 {
 	struct passwd pwent;
-
-#ifdef	SHADOWPWD
 	struct spwd spent;
-#endif
 
 	if (!oflg)
 		find_new_uid ();
@@ -1424,9 +1267,7 @@ static void usr_update (void)
 	 */
 
 	new_pwent (&pwent);
-#ifdef	SHADOWPWD
 	new_spent (&spent);
-#endif
 
 	/*
 	 * Create a syslog entry. We need to do this now in case anything
@@ -1434,7 +1275,7 @@ static void usr_update (void)
 	 */
 
 	SYSLOG ((LOG_INFO,
-		 "new user: name=%s, uid=%u, gid=%u, home=%s, shell=%s",
+		 "new user: name=%s, UID=%u, GID=%u, home=%s, shell=%s",
 		 user_name, (unsigned int) user_id,
 		 (unsigned int) user_gid, user_home, user_shell));
 
@@ -1459,27 +1300,7 @@ static void usr_update (void)
 			 _("%s: error adding new password entry\n"), Prog);
 		exit (E_PW_UPDATE);
 	}
-#ifdef NDBM
-	/*
-	 * Update the DBM files. This creates the user before the flat
-	 * files are updated. This is safe before the password field is
-	 * either locked, or set to a valid authentication string.
-	 */
 
-	if (pw_dbm_present ()) {
-		if (!pw_dbm_update (&pwent)) {
-			fprintf (stderr,
-				 _
-				 ("%s: error updating password dbm entry\n"),
-				 Prog);
-			exit (E_PW_UPDATE);
-		} else
-			pw_dbm_added = 1;
-	}
-	endpwent ();
-#endif
-
-#ifdef	SHADOWPWD
 	/*
 	 * Put the new (struct spwd) in the table.
 	 */
@@ -1490,27 +1311,6 @@ static void usr_update (void)
 			 Prog);
 		exit (E_PW_UPDATE);
 	}
-#ifdef	NDBM
-	/* 
-	 * Update the DBM files for the shadow password. This entry is
-	 * output before the entry in the flat file, but this is safe as
-	 * the password is locked or the authentication string has the
-	 * proper values.
-	 */
-
-	if (is_shadow_pwd && sp_dbm_present ()) {
-		if (!sp_dbm_update (&spent)) {
-			fprintf (stderr,
-				 _
-				 ("%s: error updating shadow passwd dbm entry\n"),
-				 Prog);
-			fail_exit (E_PW_UPDATE);
-		} else
-			sp_dbm_added++;
-		endspent ();
-	}
-#endif
-#endif				/* SHADOWPWD */
 
 	/*
 	 * Do any group file updates for this user.
@@ -1578,7 +1378,7 @@ static void create_mail (void)
 			if (access (ms, R_OK) != 0) {
 				fd = open (ms,
 					   O_CREAT | O_EXCL | O_WRONLY |
-					   O_TRUNC);
+					   O_TRUNC, 0);
 				if (fd != -1) {
 					fchown (fd, user_id, mail_gid);
 					fchmod (fd, mode);
@@ -1625,21 +1425,19 @@ int main (int argc, char **argv)
 	bindtextdomain (PACKAGE, LOCALEDIR);
 	textdomain (PACKAGE);
 
-	OPENLOG("useradd");
+	OPENLOG ("useradd");
 
-	sys_ngroups = sysconf(_SC_NGROUPS_MAX);
-	user_groups = malloc((1 + sys_ngroups) * sizeof(char *));
+	sys_ngroups = sysconf (_SC_NGROUPS_MAX);
+	user_groups = malloc ((1 + sys_ngroups) * sizeof (char *));
 
-#ifdef SHADOWPWD
-	is_shadow_pwd = spw_file_present();
-#endif
+	is_shadow_pwd = spw_file_present ();
 #ifdef SHADOWGRP
-	is_shadow_grp = sgr_file_present();
+	is_shadow_grp = sgr_file_present ();
 #endif
 
-	get_defaults();
+	get_defaults ();
 
-	process_flags(argc, argv);
+	process_flags (argc, argv);
 
 #ifdef USE_PAM
 	retval = PAM_SUCCESS;
@@ -1650,8 +1448,7 @@ int main (int argc, char **argv)
 	}
 
 	if (retval == PAM_SUCCESS) {
-		retval =
-		    pam_start ("useradd", pampw->pw_name, &conv, &pamh);
+		retval = pam_start ("useradd", pampw->pw_name, &conv, &pamh);
 	}
 
 	if (retval == PAM_SUCCESS) {
@@ -1669,30 +1466,11 @@ int main (int argc, char **argv)
 	}
 
 	if (retval != PAM_SUCCESS) {
-		fprintf (stderr, _("%s: PAM authentication failed\n"),
-			 Prog);
+		fprintf (stderr, _("%s: PAM authentication failed\n"), Prog);
 		exit (1);
 	}
-
-	OPENLOG("useradd");
 #endif				/* USE_PAM */
 
-	/*
-	 * The open routines for the NDBM files don't use read-write
-	 * as the mode, so we have to clue them in.
-	 */
-
-#ifdef	NDBM
-	pw_dbm_mode = O_RDWR;
-#ifdef	SHADOWPWD
-	sp_dbm_mode = O_RDWR;
-#endif
-	gr_dbm_mode = O_RDWR;
-#ifdef	SHADOWGRP
-	sg_dbm_mode = O_RDWR;
-#endif
-#endif
-
 	/*
 	 * See if we are messing with the defaults file, or creating
 	 * a new user.
@@ -1711,8 +1489,7 @@ int main (int argc, char **argv)
 	 */
 
 	if (getpwnam (user_name)) {
-		fprintf (stderr, _("%s: user %s exists\n"), Prog,
-			 user_name);
+		fprintf (stderr, _("%s: user %s exists\n"), Prog, user_name);
 		exit (E_NAME_IN_USE);
 	}
 
